/*-
 *
 * AvpMonitor for Linux (v0.1) 
 * Thanks Markus Wolf <klog@hert.org>, Promisc Security, who gives
 * us basic idea of interceptor.
 *
 * Copyright (C) 1999 Kaspersky Lab.
 * http://www.avp.ru
 *
 * This file is part of AvpMonitor
 * 
 * AvpMonitor is free software; you can redistribute it and/or modify
 * it without limitation.
 *
 * AvpMonitor is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with GNU CC; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.  
 * 
 */

#include <stdarg.h>

//#include <linux/types.h>
#include <asm/uaccess.h>
#include <asm/segment.h>
#include <asm/system.h>

#include <linux/errno.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/tty.h>
#include <linux/tty_driver.h>
#include <linux/malloc.h>

#define AUDIT_BUF_LEN     1024 

int fl_audit_open=0;
char *auditBuf=NULL;
unsigned long audit_size=0;
unsigned long audit_max_pos=0;
static unsigned long auditBufStart=0;
static unsigned long audit_chars  =0;
static struct wait_queue *audit_wait=NULL;

inline void init_audit_buffer(void)
 {
  if(!auditBuf)
   {                                         
    if(!(auditBuf=(char*)kmalloc(AUDIT_BUF_LEN,GFP_KERNEL))) return;
    //if(!(auditBuf=(char*)__get_free_page(GFP_USER))) return;
    audit_max_pos=PAGE_SIZE;
   }
  auditBufStart=0;
  audit_chars=0;
  audit_size=0;
 }

inline void release_audit_buffer(void)
 {
  if(auditBuf)
   {
    kfree(auditBuf); //free_page((unsigned long)auditBuf);
    auditBuf=NULL;
    audit_max_pos=0;
   }
  auditBufStart=0;
  audit_chars=0;
  audit_size=0;
 }

#ifdef AUDIT_BEFORE
// To be used to get both strings and integers 
int logpos;
int getunit(char *src, char *dst, int len)
 {
  int i;

  memset(dst,0,len);
  for(i=0;i<(len-1);logpos++,i++)
   {
    if(src[logpos]==' ') { logpos++; break; }
    if(src[logpos]==0) { logpos++; break; }
    dst[i] = src[logpos];
   }
  return (simple_strtol(dst,(char **)NULL,10));//atol(dst));
 }
#endif   

#define PNAME_LEN 18
asmlinkage int audit(const char *fmt, ...)
 {
  int res;
  va_list args;
  long flags;
  /*static*/char buf[1024];
  char *p,*buf_end;

  save_flags(flags);
  cli();

  va_start(args,fmt);
	
  res=vsprintf(buf,fmt,args); 
  buf_end=buf+res;
  va_end(args);

#ifdef AUDIT_BEFORE
  if(strncmp(buf,"111 ",4)==0) //AUDIT_OPEN_BEFORE
   {
    int  flag;
    char pname[PNAME_LEN];
    int  uid;	// There may be special chars (like '*') 
    int  pid;
    char tmp[20];

    logpos=0;

    flag=getunit(buf, tmp, sizeof(tmp));
    getunit(buf,pname, sizeof(pname));
    pid =getunit(buf, tmp, sizeof(tmp));
    uid =getunit(buf, tmp, sizeof(tmp));

    //if((strcmp(pname,"AvpDaemon")!=0)&&(strcmp(pname,"AvpLinux")!=0))
    // TestObjectByDaemon(&buf[logpos]);
   }
#endif   
  if(!auditBuf) init_audit_buffer();

  if((fl_audit_open)&&(auditBuf))
   {
    for(p=buf;p<buf_end;p++)
     {
      auditBuf[(auditBufStart+audit_size) & (AUDIT_BUF_LEN-1)] = *p;
      if(audit_size<AUDIT_BUF_LEN)
        audit_size++;
      else
       {
        auditBufStart++;
        auditBufStart&=AUDIT_BUF_LEN-1;
       }
      audit_chars++;
      if(*p=='\n') break;
     }
   }

  restore_flags(flags);
  wake_up_interruptible(&audit_wait);

  return res;
 }

asmlinkage int sys_audit(char *buf, int count)
 {
  unsigned long len;
  char c;
  int error=0;

  if(!suser()) return -EPERM;
  
  if(!count) goto done; 
  error= -EINVAL;
  if(!buf || count<0) goto done;
  error=verify_area(VERIFY_WRITE,buf,count);
  if(error) goto done;

  error=-EAGAIN;

  cli();
  while (!audit_size)
   {
//    if(current->signal & ~current->blocked)
//     {
//      sti();
//      return -ERESTARTSYS;
//     }
    interruptible_sleep_on(&audit_wait);
   }
  len=0;
  while(audit_size && len<count)
   {
    c=*((char *)auditBuf+auditBufStart);
    auditBufStart++;
    audit_size--;
    auditBufStart&=AUDIT_BUF_LEN-1;
    sti();
    put_user(c,buf);
    buf++;
    len++;
    cli();
   }
  sti();
  return len; 

done:
  return error;		
 }

