
//-------------- syslinks -----------
#include <unistd.h>
#ifndef _POSIX_SOURCE
#define _POSIX_SOURCE
#endif
#include <stdio.h>
#ifndef __FreeBSD__
#include <malloc.h>
#else
#include <stdlib.h>
#endif
#include <string.h>
#include <fcntl.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <time.h>
#include <stddef.h>
#include <errno.h>

#ifndef S_ISLNK
#define S_ISLNK(mode) (((mode) & (_S_IFMT)) == (_S_IFLNK))
#endif

#ifndef PATH_MAX
#define PATH_MAX 1024
#endif

#define SL_VERBOSE   0x0001
#define SL_FIX_LINKS 0x0002
#define SL_DELETE    0x0008
#define SL_SHORTEN   0x0010
#define SL_TESTING   0x0020

//static int sl_regim;

// tidypath removes excess slashes and "." references from a path string

static int substr(char *s,char *old,char *news)
 {
  char *tmp=NULL;
  int oldlen=strlen(old), newlen = 0;

  if (NULL==strstr(s, old)) return 0;

  if(news)newlen=strlen(news);

  if(newlen>oldlen)
   {
    if((tmp=(char*)malloc(strlen(s)))==NULL)
     {
      fprintf(stderr, "no memory\n");
      return 1; // exit (1);
     }
   }

  while(NULL != (s = strstr(s, old)))
   {
    char *p,*old_s=s;

    if(news)
     {
      if(newlen > oldlen)
      old_s = strcpy(tmp, s);
      p=news;
      while (*p) *s++ = *p++;
     }
    p=old_s + oldlen;
    while ((*s++ = *p++));
   }
  if(tmp) free(tmp);
  return 1;
 }

int TidyPath (char *path)
 {
  int tidied = 0;
  char *s, *p;

  strcat(path,"/");	// tmp trailing slash simplifies things 
  while (substr(path, "/./", "/")) tidied = 1;
  while (substr(path, "//", "/"))  tidied = 1;

  while (NULL != (p = strstr(path,"/../")))
   {
    for (s = p+3; p != path && *--p != '/';);
    if (*p != '/') break;
    if (p == path) ++p;
    while ((*p++ = *s++));
    tidied = 1;
   }
  if (!*path) strcpy(path,"/");
  p = path + strlen(path) - 1;
  if (p != path && *p == '/')
    *p-- = '\0';	/* remove tmp trailing slash */
  while (p != path && *p == '/')
   {	/* remove any others */
    *p-- = '\0';
    tidied = 1;
   }
  while (!strncmp(path,"./",2))
   {
    for (p = path, s = path+2; (*p++ = *s++););
    tidied = 1;
   }
  return tidied;
 }

char* CheckSymlink(char *path,dev_t my_dev)
 {
  static char slpath[PATH_MAX],abspath[PATH_MAX]; //,news[PATH_MAX]
  char *p;//, *lp, *tail, *msg,*np;
  struct stat stbuf;
  int c;//, fix_abs = 0, fix_messy = 0, fix_long = 0;

  if((c=readlink(path,slpath,sizeof(slpath))) == -1)
   {
    perror(path);
    return NULL;
   }
  slpath[c]='\0';	// readlink does not null terminate it

  // construct the absolute address of the link
  abspath[0]='\0';
  if(slpath[0] != '/')
   {
    strcat(abspath,path);
    if(((c=strlen(abspath)) > 0) && (abspath[c-1] == '/'))
	abspath[c-1] = '\0';
    if((p=strrchr(abspath,'/')) != NULL) *p='\0';
    strcat(abspath,"/");
   }
  strcat(abspath,slpath);
  TidyPath(abspath);

  // check for various things 
  if(stat(abspath,&stbuf)==-1)
   {
    //printf("dangling: %s -> %s\n",path,slpath);
    //if(sl_regim&&SL_DELETE)
    // {
    //  if(unlink (path))	perror(path); 
    //  else printf("deleted:  %s -> %s\n", path, lpath);
    // }
    return NULL;
   }
  //if(stbuf.st_dev != my_dev)  msg = "other_fs:";
  //else
  // if(slpath[0] == '/')     {  msg = "absolute:"; fix_abs = 1;  }
  // else
  //  if(sl_regim&&SL_VERBOSE) msg = "relative:";
  //  else                     msg = NULL;
  //  fix_messy =TidyPath(strcpy(news,lpath));
  //  if(sl_regim&&SL_SHORTEN) fix_long=ShortenPath(news,path);
  //  if(!fix_abs)
  //   {
  //    if (fix_messy) msg = "messy:   ";
  //    else           
  //      if (fix_long) msg = "lengthy: ";
  //   }
    //if(msg != NULL) printf("%s %s -> %s\n", msg, path, lpath);
    //if(!(sl_regim&&SL_FIX_LINKS || sl_regim&&SL_TESTING) || !(fix_messy || fix_abs || fix_long))
    //  return;

    //if(fix_abs)
    // {
    //	// convert an absolute link to relative: 
    //	// point tail at first part of lpath that differs from path 
    //	// point p    at first part of path  that differs from lpath 
    //	TidyPath(slpath);
    //	tail=lp=lpath;
    //	p=path;
    //	 {
    //	  if (*lp++ == '/')
    //	   {
    //	    tail = lp;
    //	    while (*++p == '/');
    //	   }
    //	 }

	// now create new, with "../"s followed by tail 
    //	np = news;
    //	while (*p)
    //	 {
    //	  if (*p++ == '/')
    //	   {
    //	    *np++ = '.';
    //	    *np++ = '.';
    //	    *np++ = '/';
    //	    while (*p == '/') ++p;
    //	   }
    //	 }
    //	strcpy (np, tail);
    //	(void) TidyPath(news);
    // }
    //if(!sl_regim&&SL_TESTING)
    // {
    //  if (unlink (path))
    //   {
    //	perror(path);
    //	return;
    //   }
    //  if(symlink(news, path))
    //   {
    //	perror(path);
    //	return;
    //   }
    // }
    //printf("changed:  %s -> %s\n", path, news);
    return abspath;
 }

