/*-
 * AvpMonitor for Linux (v0.1) 
 * Thanks Markus Wolf <klog@hert.org>, Promisc Security, who gives
 * us basic idea of interceptor.
 *
 * Copyright (C) 1999 Kaspersky Lab.
 * http://www.avp.ru
 *
 * This file is part of AvpMonitor
 * 
 * AvpMonitor is free software; you can redistribute it and/or modify
 * it without limitation.
 *
 * AvpMonitor is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with GNU CC; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.  
 */

//#include <malloc.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <ctype.h>

#include "common.h"
#include "../audit.h" 
#include "profile.h"
#include "report.h"

char GetStr(char *par,char *in,char **out)
 {
  if(strncasecmp(in,par,strlen(par))) return 0;
  in+=strlen(par);
  while(*in==' ') (*in)++;
  if(*in++!='=') return 0;
  while(*in==' ') in++;
  if(!strlen(in)) return 1;
  if(*out) free(*out);
  // if((*out=strdup(in))==NULL)
  if((*out=(char *)malloc(strlen(in)+1))==NULL) return -1;
  if(strcpy(*out,in)==NULL) { /*prg_error=1; exit_();*/ return -1; }
  if(!*out) prf_global_state=PERR_MEM;
  return 1;
 }

char xget_str(char *par, char *in, char *out, int lim)
 {
  if(strncasecmp(in,par,strlen(par))) return 0;
  in+=strlen(par);
  while(*in==' ') (*in)++;
  if(*in++!='=') return 0;
  while(*in==' ') in++;
  if(!strlen(in)) return 1;
  if(strlen(in)>(size_t)lim) return 0;
  strcpy(out,in);
  return 1;
 } 

char xxget_str(char *par, char *in, char *out,int lim)
 {
  if(strncasecmp(in,par,strlen(par))) return 0;
  in+=strlen(par);
  while(*in==' ') (*in)++;
  if(*in++!='=') return 0;
  while(*in==' ') in++;
  if(!strlen(in)) return 1;
  if(strlen(in)>(size_t)lim) return 0;
  while(*in) if(*in>='0'&&*in<='9') { *out++=*in; in++; }
  *out=0;
  return 1;
 }

char get_bool(char *par,char *in, char *out)
 {
  if(strncasecmp(in,par,strlen(par))) return 0;
  in+=strlen(par);
  while(*in==' ') (*in)++;
  if(*in++!='=') return 0;
  while(*in==' ') in++;
  if(!strcasecmp(in,"Yes")) *out=1;
  else
   if(!strcasecmp(in,"No")) *out=0;
   else prf_global_state=PERR_COR;
  return 1;
 }

char set_bool_msk(char *par,char *in,ULONG *out,ULONG ulMask)
 {
  if(strncasecmp(in,par,strlen(par))) return 0;
  in+=strlen(par);
  while(*in==' ') (*in)++;
  if(*in++!='=') return 0;
  while(*in==' ') in++;
  if(!strcasecmp(in,"Yes")) *out|=ulMask;
  else
   if(!strcasecmp(in,"No")) *out&=~ulMask;
   else prf_global_state=PERR_COR;
  return 1;
 }

char get_atoi(char *par, char *in, char *out, char max)
 {
  if(strncasecmp(in,par,strlen(par))) return 0;
  in+=strlen(par);
  while(*in==' ') (*in)++;
  if(*in++!='=') return 0;
  if(*(in+1)||*in<'0'||*in>max+'0') return 1;
  *out=(char)(*in-'0');
  return 1;
 }

ProfileData prf_data; 
unsigned char prf_global_state; 

void SetDefaultOptions(void)
 {
  prf_data.TestMask=MSK_AUDIT_OPEN_AFTER;
  bzero(&prf_data.AtRunLog,sizeof(rule));
  bzero(&prf_data.FSOpenLog,sizeof(rule));
  bzero(&prf_data.ModInstLog,sizeof(rule));
  bzero(&prf_data.NetOutLog,sizeof(rule));
  bzero(&prf_data.NetInLog,sizeof(rule));
  bzero(&prf_data.LogUserLog,sizeof(rule));
  bzero(&prf_data.SetUIdLog,sizeof(rule));
  prf_data.Symlinks=0;
    // -LH symbolic links on the command line are followed (Symbolic
    //     links encountered in the tree traversal are not followed).
    // -LL all symbolic links are followed.
    // -LP no symbolic links are followed.
  prf_data.Directories=0;
  prf_data.CrossFs=1; // If true, don't cross filesystem boundaries.
  prf_data.ExcludeFiles=0;
  prf_data.ExcludeDir=NULL;
  strcpy(prf_data.ExcludeMask,"*.txt;*.cmd");
  prf_data.CommonLog=1;
  strcpy(prf_data.CommonLogFileName,"audit.log");
  prf_data.CommonLogfd=-1;
  prf_data.Append=0;
  *(prf_data.LogFileSize)=0;
  prf_data.LogFileLimit=0;
  prf_data.Warnings=1;
  prf_data.ShowOK=0;
  prf_data.OtherMessages=1;
  prf_data.UseTime=1;
 }

ULONG SearchFile(char* sFileName,char* sDestName,char* sExt)
 {
  if(strlen(sExt)>0)
   if(!strchr(sFileName,'.') || (strrchr(sFileName,'.')<strrchr(sFileName,'/')) )
     strcat(sFileName,sExt);
  if(access(sFileName,0)==-1)
   {
    strcpy(sDestName,prg_path);
    strcat(sDestName,sFileName);
    if(access(sDestName,0)==-1)
     {
      strcpy(sDestName,start_path);
      strcat(sDestName,sFileName);
      if(access(sDestName,0)==-1) strcpy(sDestName,sFileName);
     }
   }
  else
    strcpy(sDestName,sFileName);

  return 0;
 }

int confpos=0;

int getelem(char *src, char *dst, int len)
 {
  int i;

  bzero(dst,len);
  for(i=0;i<(len-1);confpos++,i++)
   {
    if((src[confpos]==':')||(src[confpos]==0) ) { confpos++; break; }
    dst[i]=src[confpos];
   }
  return (atol(dst));
 }

int parse_rule(char *buf,rule* rules)
 {
  char filename[256];
  char flOpen=!prf_data.CommonLog;

  confpos=0;
  bzero(rules,sizeof(rule));
  getelem(buf,rules->flags,FLAGS_LEN);
  getelem(buf,rules->pname,PNAME_LEN);
  getelem(buf,rules->pid,UID_LEN);
  getelem(buf,rules->uid,PID_LEN);
  getelem(buf,filename,256);

  if(strcmp(rules->log.filename,filename)==0) return rules->log.logfd;

  strcpy(rules->log.filename,filename);
  //rules->log.logfd=fopen(filename,"a+");
  if(!prf_data.CommonLog) rules->log.logfd=reportInit(&flOpen,filename);
  if(rules->log.logfd==-1) //==NULL)
   {
    char szErrMess[100];
    sprintf(szErrMess,"Cannot open log(%s)",filename);
    die(szErrMess);
   }    

  LOG((1,"Log format: flags(%s) pname(%s) pid(%s) uid(%s)\n",rules->flags,rules->pname,rules->pid,rules->uid));
  LOG((1,"Log filename: %s(%d) \n",filename,rules->log.logfd==(-1) ? 0:1));
  return rules->log.logfd;
 }

ULONG LoadProfile(char *name)
 {
  int   fl;
  char  section = 0xff,*buf,*next;
  unsigned z;
  char  sectstr[]={"AtRun]\0FSOpen]\0ModInst]\0NetOut]\0NetIn]\0LogUser]\0SetUId]\0Options]\0Report]\0Customize]\0"};
  char  secofs[] ={0,7,15,24,32,39,48,56,65,73};
  char  dname[0x200];
  char  Rulbuf[0x200];

  SearchFile(name,dname,".conf");
  prf_global_state=0;

  if((fl=open(dname,O_RDONLY))==-1) return PERR_OPN; 
  z=lseek(fl,0,SEEK_END);
  lseek(fl,0,SEEK_SET);
  buf=(char *)malloc(z);
  if(!buf) return PERR_MEM;
  if(read(fl,(void *)buf,z)==-1)
   {
    close(fl);
    free(buf);
    return PERR_DSK;
   }
  if(close(fl)==-1)
   {
    free(buf);
    return PERR_DSK;
   }
  // ok, now parse file in mem
  next=strtok(buf,"\r\n");
  while(next)
   {
    while(*next==' ') next++;
    if(*next=='[')
     {
      next++;
      int fl;
      section=0xff;
      for(fl=0;fl<10;fl++)
       if(!strcasecmp(next,sectstr+secofs[fl]))
        {section =(char)fl; break;}
     }
    else
     switch(section)
     {
      case 0: //[AtRun]
              if(xget_str("Log",next,Rulbuf,MAXLOGNAMELEN-1))
	        { parse_rule(Rulbuf,&prf_data.AtRunLog); break; }
              if(set_bool_msk("TestBefore",next,&prf_data.TestMask,MSK_AUDIT_EXEC_BEFORE)) break;
              if(set_bool_msk("TestAfter",next,&prf_data.TestMask,MSK_AUDIT_EXEC_BEFORE)) break;
              break;
      case 1: //[FSOpen]
              if(xget_str("Log",next,Rulbuf,MAXLOGNAMELEN-1))
	        { parse_rule(Rulbuf,&prf_data.FSOpenLog); break; }
              if(set_bool_msk("TestBefore",next,&prf_data.TestMask,MSK_AUDIT_OPEN_BEFORE)) break;
              if(set_bool_msk("TestAfter",next,&prf_data.TestMask,MSK_AUDIT_OPEN_AFTER)) break;
              break;
      case 2: //[ModInst]
              if(xget_str("Log",next,Rulbuf,MAXLOGNAMELEN-1))
	        { parse_rule(Rulbuf,&prf_data.ModInstLog); break; }
              break;
      case 3: //[NetOut]
              if(xget_str("Log",next,Rulbuf,MAXLOGNAMELEN-1))
	        { parse_rule(Rulbuf,&prf_data.NetOutLog); break; }
              break;
      case 4: //[NetIn]
              if(xget_str("Log",next,Rulbuf,MAXLOGNAMELEN-1))
	        { parse_rule(Rulbuf,&prf_data.NetInLog); break; }
              break;
      case 5: //[LogUser]
              if(xget_str("Log",next,Rulbuf,MAXLOGNAMELEN-1))
	        { parse_rule(Rulbuf,&prf_data.LogUserLog); break; }
              break;
      case 6: //[SetUId]
              if(xget_str("Log",next,Rulbuf,MAXLOGNAMELEN-1))
	        { parse_rule(Rulbuf,&prf_data.SetUIdLog); break; }
              break;
      case 7: //[Options]
              if(get_atoi("Symlinks",next,&(prf_data.Symlinks),2)) break;
              if(get_bool("Directories",next,&(prf_data.Directories))) break;
              if(get_bool("CrossFs",next,&(prf_data.CrossFs))) break;
              if(get_bool("ExcludeFiles",next,&(prf_data.ExcludeFiles))) break;
              if(xget_str("ExcludeMask",next,prf_data.ExcludeMask,80)) break;
              if(GetStr("ExcludeDir",next,&(prf_data.ExcludeDir))) break;
              break;
      case 8: //[Report] - first section
              if(get_bool("CommonLog",next,&(prf_data.CommonLog))) break;
              //if(get_bool("ExtReport",next,&(prf_data.ExtReport))) break;
              if(xget_str("CommonLogFileName",next,prf_data.CommonLogFileName,MAXLOGNAMELEN-1)) break;
              if(get_bool("Append",next,&(prf_data.Append))) break;
              if(xxget_str("LogFileSize",next,prf_data.LogFileSize,7)) break;
              if(get_bool("LogFileLimit",next,&(prf_data.LogFileLimit))) break;
              break;
      case 9 ://[Customize]
              if(get_bool("Warnings",next,&(prf_data.Warnings))) break;
              if(get_bool("ShowOK",next,&(prf_data.ShowOK))) break;
              if(get_bool("OtherMessages",next,&(prf_data.OtherMessages))) break;
              if(get_bool("UseTime",next,&(prf_data.UseTime))) break;
              //if(get_bool("Sound",next,&(prf_data.Sound))) break;
              break;
     } // switch (section)
    if(prf_global_state != PERR_MEM) next = strtok(0,"\r\n"); else next = 0;
   }
  free(buf);

  //  remove_space_((unsigned char*)prf_data.UserMask);
  return prf_global_state;
 }

