// Program to test daemon scanning.

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>        // struct sockaddr_un 
#include <sys/file.h>
#include <time.h>
#include <stdarg.h>
#if !(defined (sun) && defined(i386))
  #include <paths.h>
#else  
 #include <fcntl.h>
 #include <strings.h>
#endif
#include <errno.h>
extern int errno;

//#define DEBUG_MESSAGES
#ifdef DEBUG_MESSAGES
 #define dprintf(args) printf args
#else
 #define dprintf(args) 
#endif

#ifdef linux
#define AVP_NAME    "Antiviral Toolkit Pro for Linux (Cpp client for Daemon)"
#else
 #ifdef bsdi
  #define AVP_NAME    "Antiviral Toolkit Pro for BSDi (Cpp client for Daemon)"
 #else
  #ifdef __FreeBSD__ 
   #define AVP_NAME    "Antiviral Toolkit Pro for FreeBSD (Cpp client for Daemon)"
  #else    
   #if defined (sun) && defined(i386)
    #define AVP_NAME    "Antiviral Toolkit Pro for Solaris (Cpp client for Daemon)"
   #else    
    #define AVP_NAME    "Antiviral Toolkit Pro for Unix (Cpp client for Daemon)"
   #endif
  #endif
 #endif
#endif
#define AVP_DATE "built on "__DATE__" at "__TIME__ " "

//#define FULLLINE
#ifndef FULLLINE
 typedef unsigned long ULONG;
 #define FL_GETVERSION  0x04
#endif

// ... Cfg pathnames 
#define AVP_NODE_DIR      "/var/run"
#define AVP_NODE_DIR_MODE 0777
#define AVP_NODE_PID      AVP_NODE_DIR "/AvpPid"
#define AVP_NODE_LOG      AVP_NODE_DIR "/AvpLog"
#define AVP_NODE_CTL      AVP_NODE_DIR "/AvpCtl\0\0\0"
char *NodePid=AVP_NODE_PID;
char *NodeCtl=AVP_NODE_CTL;

static int  AvpFile=-1;		// fd for log 
static int  connected;		 // have done connect 

struct sockaddr_un AvpTestAddr;	// AF_UNIX address of local logger 
// start test
int AvpTestStart()
 {
  if(AvpFile==-1)
   {
    bzero((char *)&AvpTestAddr,sizeof(AvpTestAddr));
    AvpTestAddr.sun_family=AF_UNIX;
    strcpy(AvpTestAddr.sun_path,NodeCtl);
    if((AvpFile=socket(AF_UNIX,SOCK_STREAM,0))<0)
      printf("create socket error: socket() not created\n");
    // fcntl(AvpFile,F_SETFD,1); 
   }

  if(AvpFile!=-1 && !connected &&
#ifdef linux
    connect(AvpFile,(struct sockaddr *)(&AvpTestAddr),sizeof(AvpTestAddr.sun_family)+strlen(NodeCtl))>=0)
#else
    connect(AvpFile,(struct sockaddr *)(&AvpTestAddr),sizeof(AvpTestAddr.sun_family)+strlen(NodeCtl)+1)>=0)
#endif    
   connected=1;
  if(connected==1)
   {
    dprintf(("AvpTestStart connected to AvpDaemon\n")); 
    return 0;
   }    
  printf("AvpTestStart cannot connected to AvpDaemon\n"); 
  return -1;
 }

// close the testing
void AvpTestClose()
 {
  (void)close(AvpFile);
  AvpFile=-1;
  connected=0;
  dprintf(("AvpTestClose. connected=%d\n",connected)); 
 }

int AvpTestByDaemon(int flags,char *buftoscan,ULONG *ulFlags,ULONG* buflen)
 {
  register int len=strlen(buftoscan);

  // output the message to the local logger 
  dprintf(("write string (%s) to AvpDaemon\n",buftoscan));

  if(write(AvpFile,buftoscan,len+1)>=0)
   {
    int Rez;
    long uintbuf=0; 
    char *ResultBuf=NULL;
waitrez:
    dprintf(("Wait results: \n"));
    if((Rez=read(AvpFile,(char*)&uintbuf,2))==-1) return -2;
    int ExitCode=uintbuf&0xff-0x30;
    if((uintbuf&0x000f)!=0xf) //0x3f '?'
     printf("Test result: %x\n",ExitCode);
    else 
     dprintf(("Disinfect queries: \n"));
    dprintf(("Test result: 0x%x, flags: 0x%x\n",uintbuf&0x00ff,uintbuf&0xff00));
    ResultBuf=NULL;
    if((uintbuf&0xff00)!=0)
     {
      if((uintbuf&0x200)!=0)
       {
        if((Rez=read(AvpFile,(char*)buflen,sizeof(ULONG)))==-1) return -2;
	*ulFlags|=1;
       }
      if((uintbuf&0x100)!=0)
       {
        if((Rez=read(AvpFile,(char*)&uintbuf,sizeof(ULONG)))==-1) return -2;
        dprintf(("Result string lenght: %d\n",uintbuf));
        ResultBuf=(char*)malloc(uintbuf+1);
        if(ResultBuf!=NULL)
	 {
          ResultBuf[0]=0;
          char *ResultStr=ResultBuf;
          //if((Rez=recv(AvpFile,ResultStr,uintbuf,0))==-1) return -2;
          while((uintbuf>0)&&((Rez=recv(AvpFile,ResultStr,uintbuf,0))!=0))
           if(Rez==-1)
            { dprintf(("Client cannot reaf from AvpDaemon!")); return -2; }
           else 
            {
             uintbuf-=Rez;
             ResultStr[Rez]=0;
  	     ResultStr+=Rez;
            }
         }	    
       }
     }  
    switch (ExitCode&0x0f)
     {
      case 8:	
        printf("Corrupted objects were found\n");
        break;
      case 7:	
        printf("File AvpScanner is corrupted\n");
        break;
      case 6:	
        printf("All viruses deleted\n");
        break;
      case 5:	
        printf("All viruses disinfected\n");
        break;
      case 4:	
        printf("Known viruses were detected\n");
        break;
      case 3:	
        printf("Suspicious objects were found\n");
        break;
      case 2:	
        printf("Warning\n");
        break;
      case 1:	
        printf("Virus scan was not complete\n");
        break;
      case 0:	
        printf("No viruses were found\n");
        break;
      case 0xf:
        printf("%s",ResultBuf);
        if(ResultBuf!=NULL) free(ResultBuf);
        char dlgres;
        dlgres=getchar();
        if(dlgres!='\n') while(!feof(stdin) && (fgetc(stdin)!='\n')) ;
        int rezs;
        if((rezs=send(AvpFile,(char*)&dlgres,1,0))==-1)
          printf("Client can not write disinfect dialog result to AvpDaemon");
        goto waitrez;
      default:	
        printf("Error!(test result %d)\n",Rez);
        break;
     }
    switch (ExitCode&0xf0)
     {
      case 8:	
        printf("Internal error: Integrity failed.\n");
        break;
      case 4:	
        printf("Internal error: Bases not found.\n");
        break;
     }
    printf("Found viruses: %s\n",ResultBuf);
    if(ResultBuf!=NULL) free(ResultBuf);

    return ExitCode;
   }
  return -1;
 }

int AvpTest(int flags,const char *fmt, ...)
 {
  int rez=-1;

  if(AvpFile<0 || !connected)
   if(AvpTestStart()!=0) return -1;

  va_list ap;
  va_start(ap, fmt);

  register char *p;
  int saved_errno=0;
  char tbuf[2048], fmt_cpy[1024];
  time_t now;

  // build the message 
  (void)time(&now);
  (void)sprintf(tbuf,"<%d>%.15s:",flags,ctime(&now)+4);
  for(p=tbuf;*p;++p) ;

  // substitute error message for %m 
   {
    register char ch, *t1, *t2;

    for(t1=fmt_cpy;(ch=*fmt)!='\0' && t1<fmt_cpy+sizeof(fmt_cpy);++fmt)
     if(ch=='%'&& fmt[1]=='m')
      {
       ++fmt;
       for(t2=strerror(saved_errno);(*t1 = *t2++);++t1);
      }
     else
      *t1++ =ch;
    *t1='\0';
   }
  (void)vsprintf(p,fmt_cpy,ap);

  ULONG ulFlags=0,ulDiffer=0;
  rez=AvpTestByDaemon(flags,tbuf,&ulFlags,&ulDiffer);
  switch (rez)
   {
    case -1:
      printf("Error: cannot write to AvpDaemon!\n");
      break;
    case -2:
      printf("Error: cannot read from AvpDaemon!\n");
      break;
   }
  va_end(ap);
  return rez;
 }

int main(int argc,char *argv[])
 {
  int rez=-1;

  for(unsigned char z=1;z<argc;z++)
   {
    if(*argv[z]=='-')
     switch (argv[z][1])
     {
      case 'f': // -f=path path for AvpCtl file 
        if((strlen(argv[z])>3) && (argv[z][2]=='='))
         {
         // if(NodeCtl!=NULL) free(NodeCtl);
          NodeCtl=(char*)malloc(strlen(&argv[z][3])+strlen("AvpCtl\0\0\0")+10);
          strncpy(NodeCtl,&argv[z][3],strlen(&argv[z][3]));
          NodeCtl[strlen(&argv[z][3])]=0;
          if(NodeCtl[strlen(NodeCtl)-1]!='/') strcat(NodeCtl,"/");
          strcat(NodeCtl,"AvpCtl\0\0\0");

          NodePid=(char*)malloc(strlen(&argv[z][3])+strlen("AvpPid")+10);
          strncpy(NodePid,&argv[z][3],strlen(&argv[z][3]));
          NodePid[strlen(&argv[z][3])]=0;
          if(NodePid[strlen(NodePid)-1]!='/') strcat(NodePid,"/");
          strcat(NodePid,"AvpPid");
	 }	
        break;
     }
   }

  if(AvpTestStart()==0)
   if(argc>1)
    {
     for(unsigned char z=1;z<argc;z++)
      {
       if(*argv[z]=='-')
        switch (argv[z][1])
         {
          case 'v':
	    printf(AVP_NAME ", " AVP_DATE "\n");
            rez=AvpTest(FL_GETVERSION,"");
            AvpTestClose();

            return(rez);
         }
      }
#ifdef FULLLINE
     char CmdOpts[0x50];
     char CmdPath[0x200];
     CmdOpts[0]=0;
     CmdPath[0]=0;
     for(unsigned char z=1;z<argc;z++)
      { 
       if(*argv[z]=='-')
        {
         if((strlen(CmdOpts)+strlen(argv[z]))<0x50)
          { strcat(CmdOpts,&argv[z][1]); strcat(CmdOpts,"|");}
        }
       else
        {
         if((strlen(CmdPath)+strlen(argv[z])+2)<0x200) 
          { strcat(CmdPath,argv[z]); strcat(CmdPath,":");}
        }
      }

     if((strlen(CmdOpts)+strlen(CmdPath))>0)
      {
       // now we can put the options 
       char *buf;
       buf=(char *)malloc(strlen(CmdOpts)+strlen(CmdPath)+3);
       if(buf)
        {
         strcpy(buf,"\xfe");
         strcat(buf,CmdOpts);
         strcat(buf,"\xfe");
         strcat(buf,CmdPath);
         rez=AvpTest(0,buf);
         free(buf);
        } 
      }    
#else
     if( (*argv[1]=='-') && (*(argv[1]+1)=='\0') )
      {
       auto char *nl,bufr[512];
       auto int tested=0;
       while (!feof(stdin))
	if(fgets(bufr, sizeof(bufr), stdin) !=(char *) 0 )
	 {
	  if( (nl=strrchr(bufr,'\n'))!=(char *)0) *nl='\0';
  	  rez=AvpTest(0,bufr);
	  tested+=strlen(bufr);
	  if(tested>1024)
	   {
	    sleep(1);
	    tested=0;
	   }
	 }
      }
     else
      while(argc-- >1)
       rez=AvpTest(0,argv++[1]);
#endif
    }
   else
    printf("Not object to test.\n");

  AvpTestClose();

  printf("Return code: %d\n",rez);
  return(rez);
 }

