#ifndef __TCollection_H__
#define __TCollection_H__

#define NO_TV_STREAMS

#define Uses_TNSCollection
#define Uses_TCollection

#include <limits.h>

enum Boolean { False, True };

typedef int ccIndex;
typedef Boolean (*ccTestFunc)( void *, void * );
typedef void (*ccAppFunc)( void *, void * );

const int ccNotFound = -1;

const int maxCollectionSize=(int)(( (long) UINT_MAX - 16)/sizeof( void * ));

#if defined( Uses_TNSCollection ) && !defined( __TNSCollection )

class TNSCollection
// : public TObject
 {
public:
    TNSCollection( ccIndex aLimit, ccIndex aDelta );
    ~TNSCollection();

    virtual void shutDown();
    void *at( ccIndex index );
    virtual ccIndex indexOf( void *item );

    void atFree( ccIndex index );
    void atRemove( ccIndex index );
    void remove( void *item );
    void removeAll();
    void free( void *item );
    void freeAll();

    void atInsert( ccIndex index, void *item );
    void atPut( ccIndex index, void *item );
    virtual ccIndex insert( void *item );

#ifndef __UNPATCHED
    virtual void error( ccIndex code, ccIndex info );
#else
    static void error( ccIndex code, ccIndex info );
#endif

    void *firstThat( ccTestFunc Test, void *arg );
    void *lastThat( ccTestFunc Test, void *arg );
    void forEach( ccAppFunc action, void *arg );

    void pack();
    virtual void setLimit( ccIndex aLimit );

    ccIndex getCount()
        { return count; }

protected:
    TNSCollection();

    void **items;
    ccIndex count;
    ccIndex limit;
    ccIndex delta;
    Boolean shouldDelete;

private:
    virtual void freeItem( void *item );
};

#endif  // Uses_TNSCollection

#if defined( Uses_TCollection ) && !defined( __TCollection )

class TCollection : public virtual TNSCollection
#ifndef NO_TV_STREAMS
                                                , public TStreamable
#endif  // ifndef NO_TV_STREAMS
 {
public:
    TCollection( ccIndex aLimit, ccIndex aDelta )
        { delta = aDelta; setLimit( aLimit ); }

#ifndef NO_TV_STREAMS
private:
    virtual const char *streamableName() const
        { return name; }

    virtual void *readItem( ipstream& ) = 0;
    virtual void writeItem( void *, opstream& ) = 0;


protected:
    TCollection( StreamableInit );
    virtual void *read( ipstream& );
    virtual void write( opstream& );

public:
    static const char * const name;
#endif  // ifndef NO_TV_STREAMS
};

#ifndef NO_TV_STREAMS
inline ipstream& operator >> ( ipstream& is, TCollection& cl )
    { return is >> (TStreamable&)cl; }
inline ipstream& operator >> ( ipstream& is, TCollection*& cl )
    { return is >> (void *&)cl; }

inline opstream& operator << ( opstream& os, TCollection& cl )
    { return os << (TStreamable&)cl; }
inline opstream& operator << ( opstream& os, TCollection* cl )
    { return os << (TStreamable *)cl; }
#endif  // ifndef NO_TV_STREAMS

#endif  // TCollection

#endif  

