// Program to test daemon scanning.

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>        // struct sockaddr_un 
#include <sys/file.h>
#include <time.h>
#include <stdarg.h>
#include <paths.h>
#include <errno.h>
#include "debug.h"
#include "workstat.h"

#include "TCollection.h"
TCollection* Objects=NULL;
typedef struct
 {
  unsigned long ulLen;
  char* usBuf;
 } _mempart;

extern int errno;

// Shared memory
#include "shmem.h"
#ifdef USESHMEM
 MemForUse *ShMem;
 int semid;
#else
 typedef unsigned long ULONG;
#endif
// Shared memory definition end

// ... Cfg pathnames 
#define AVP_NODE_DIR      "/var/run"
#define AVP_NODE_DIR_MODE 0777
#define AVP_NODE_PID      AVP_NODE_DIR "/AvpPid"
#define AVP_NODE_LOG      AVP_NODE_DIR "/AvpLog"
#define AVP_NODE_CTL      AVP_NODE_DIR "/AvpCtl\0\0\0"
char *NodePid=AVP_NODE_PID;
char *NodeCtl=AVP_NODE_CTL;

static int  AvpFile=-1;		// fd for log 
static int  connected;		 // have done connect 

struct sockaddr_un AvpTestAddr;	// AF_UNIX address of local logger 
// start test
int AvpTestStart(void)
 {
  if(AvpFile==-1)
   {
    bzero((char *)&AvpTestAddr,sizeof(AvpTestAddr));
    AvpTestAddr.sun_family=AF_UNIX;
    strcpy(AvpTestAddr.sun_path,NodeCtl);
    if((AvpFile=socket(AF_UNIX,SOCK_STREAM,0))<0)
      printf("create socket error: socket() not created\n");
    // fcntl(AvpFile,F_SETFD,1); 
   }

  if(AvpFile!=-1 && !connected &&
#ifdef linux
    connect(AvpFile,(struct sockaddr *)(&AvpTestAddr),sizeof(AvpTestAddr.sun_family)+strlen(NodeCtl))>=0)
#else
    connect(AvpFile,(struct sockaddr *)(&AvpTestAddr),sizeof(AvpTestAddr.sun_family)+strlen(NodeCtl)+1)>=0)
#endif  
   connected=1;
  if(connected==1)
   {
    dprintf(("I connect with AvpDaemon\n")); 
    return 0;
   }    
  printf("I cannot connect with AvpDaemon\n"); 
  return -1;
 }

// close the testing
void AvpTestClose(void)
 {
  (void)close(AvpFile);
  AvpFile=-1;
  connected=0;
  dprintf(("AvpTestClose. connected=%d\n",connected)); 
 }

int AvpTestByDaemon(int flags,char *buftoscan,ULONG *ulFlags,ULONG* buflen)
 {
  register int len;
  register char *p;
  int fd;

  len=strlen(buftoscan);

  // output the message to the local logger 
  dprintf(("write string (%s) to AvpDaemon\n",buftoscan));

  if(write(AvpFile,buftoscan,len+1)>=0)
   {
    int Rez,uintbuf=0; 
    char *ResultBuf;
waitrez:
    dprintf(("Wait results: \n"));
    if((Rez=read(AvpFile,(char*)&uintbuf,2))==-1) return -2;
    int ExitCode=uintbuf&0xff-0x30;
    if((uintbuf&0x000f)=='?') //0x3f
     printf("Disinfect queries: \n");
    else 
     printf("Test result: %x\n",ExitCode);
    dprintf(("Test result: 0x%x, flags: 0x%x\n",uintbuf&0x00ff,uintbuf&0xff00));
    ResultBuf=NULL;
    if((uintbuf&0xff00)!=0)
     {
      if((uintbuf&0x200)!=0)
       {
        if((Rez=read(AvpFile,(char*)buflen,sizeof(ULONG)))==-1) return -2;
	*ulFlags|=1;
       }
      if((uintbuf&0x100)!=0)
       {
        if((Rez=read(AvpFile,(char*)&uintbuf,sizeof(ULONG)))==-1) return -2;
        ResultBuf=(char*)malloc(uintbuf+1);
        ResultBuf[0]=0;
        char *ResultStr=ResultBuf;
        dprintf(("Result string lenght: %d\n",uintbuf));
        //if((Rez=recv(AvpFile,ResultStr,uintbuf,0))==-1) return -2;
        while((uintbuf>0)&&((Rez=recv(AvpFile,ResultStr,uintbuf,0))!=0))
         if(Rez==-1)
          { dprintf(("Client cannot reaf from AvpDaemon!")); return -2; }
         else 
          {
            uintbuf-=Rez;
            ResultStr[Rez]=0;
  	    ResultStr+=Rez;
          }
       }
     }  
    switch (ExitCode&0x0f)
     {
      case 8:	
        printf("Corrupted objects were found\n");
        IncWorkStat(8);
        break;
      case 7:	
        printf("File AvpLinux is corrupted\n");
        IncWorkStat(7);
        break;
      case 6:	
        printf("All viruses deleted\n");
        IncWorkStat(6);
        break;
      case 5:	
        printf("All viruses disinfected\n");
        IncWorkStat(5);
        break;
      case 4:	
        printf("Known viruses were detected\n");
        IncWorkStat(4);
        break;
      case 3:	
        printf("Suspicious objects were found\n");
        IncWorkStat(3);
        break;
      case 2:	
        printf("Warning\n");
        IncWorkStat(2);
        break;
      case 1:	
        printf("Virus scan was not complete\n");
        IncWorkStat(1);
        break;
      case 0:	
        printf("No viruses were found\n");
        IncWorkStat(0);
        break;
      case 0xf:
        printf("%s",ResultBuf);
        free(ResultBuf);
        char dlgres;
        dlgres=getchar();
        if(dlgres!='\n') while(!feof(stdin) && (fgetc(stdin)!='\n')) ;
        int rezs;
        if((rezs=send(AvpFile,(char*)&dlgres,1,0))==-1)
          printf("Client can not write disinfect dialog result to AvpDaemon");
        goto waitrez;
      default:	
        printf("Error!(test result %d)\n",Rez);
        break;
     }
    switch (ExitCode&0xf0)
     {
      case 8:	
        printf("Internal error: Integrity failed.\n");
        break;
      case 4:	
        printf("Internal error: Bases not found.\n");
        break;
     }
    printf("Found viruses: %s\n",ResultBuf);
    free(ResultBuf);

    return ExitCode;
   }
  return -1;

  // output the message to the console; don't worry about
  // blocking, if console blocks everything will.
  if((fd=open(_PATH_CONSOLE,O_WRONLY|O_NOCTTY,0))<0) return -1;
  (void)strcat(buftoscan,"\r\n");
  len+=2;
  p=index(buftoscan,'>')+1;
  (void)write(fd,p,len-(p-buftoscan));
  (void)close(fd);
  return -1;
 }

int AvpTest1(int flags,const char *fmt, ...)
 {
  int ret=-1;

  if(AvpFile<0 || !connected)
   if(AvpTestStart()!=0) return -1;

  va_list ap;
  va_start(ap, fmt);

  register char *p;
  int saved_errno=0;
  char tbuf[2048], fmt_cpy[1024];
  time_t now;

  // build the message 
  (void)time(&now);
  (void)sprintf(tbuf,"<%d>%.15s:",flags,ctime(&now)+4);
  for(p=tbuf;*p;++p) ;

  // substitute error message for %m 
   {
    register char ch, *t1, *t2;

    for(t1=fmt_cpy;(ch=*fmt)!='\0' && t1<fmt_cpy+sizeof(fmt_cpy);++fmt)
     if(ch=='%'&& fmt[1]=='m')
      {
       ++fmt;
       for(t2=strerror(saved_errno);(*t1 = *t2++);++t1);
      }
     else
      *t1++ =ch;
    *t1='\0';
   }
  (void)vsprintf(p,fmt_cpy,ap);

#ifdef USESHMEM
  key_t key=IPC_PRIVATE;
  int shmid=-1;
  char *filename=(char*)malloc(strlen(p)+1);
  strcpy(filename,p);
  if((flags&(FL_USESHMEM|FL_DOUNTUSESEM))!=0)
   {
    register FILE *hFile=NULL; 
    ULONG ulFileLen=0;
    if(!((strlen(p)==0) && ((flags&FL_GETVERSION)!=0)) )
     {
      if(strcmp(filename,"-")==0)
       {
	Objects=new TCollection(10,0);

        hFile=stdin;
	int bytes_read;
	char buf[0x201];
        while ((bytes_read=fread(buf,1,0x200,hFile))>0)
         {
          _mempart *temp;
	  temp=(_mempart *)malloc(sizeof(_mempart));
	  temp->ulLen=bytes_read;
          temp->usBuf=new char[bytes_read+1];
          memcpy(temp->usBuf,buf,bytes_read);
          Objects->atInsert(Objects->getCount(),(void*)temp);
          ulFileLen+=bytes_read;
         }

        if(ferror(hFile))
         {
          //error (0, errno, "%s", file);
          if(strcmp(filename,"-")!=0) fclose(hFile);
          return -1;
         }

        if((strcmp(filename,"-")!=0) && fclose(hFile)==EOF)
         {
          //error(0, errno, "%s", file);
          return -1;
         }
       }
      else
       {
        struct stat statbuf;
        if(lstat(p,&statbuf)==-1)
         {
          dprintf(("Cannot find object(%s)\n",p));
          return -3;
         }    
        ulFileLen=statbuf.st_size;
        if(!S_ISREG(statbuf.st_mode))
         {
          dprintf(("%s is not a file\n",p));
          return -3;
         }    
   
        if((hFile=fopen(p,"r"))==NULL)
         {
          dprintf(("Error in open(%s)\n",p));
          return -3;
         }
       }
     }      

    char *keyname=NULL; 
    if(access(NodeCtl,0)!=-1) keyname=NodeCtl;
    else
     {
      perror("could not find program");
      free(filename);
      if(strcmp(filename,"-")!=0) fclose(hFile);
      return -4;
     }     

    char chPrj='D';
    if((key=ftok(keyname,chPrj))<0) 
     { 
      printf("umpossible take a key\n");
      free(filename);
      if(strcmp(filename,"-")!=0) fclose(hFile);
      return -4;
     }
    dprintf(("key: <%x> keyname:<%s>\n",key,keyname));

    // create sharedmem buf
    if((shmid=shmget(key,(sizeof(MemForUse)+ulFileLen+(MemAlignment-1)) & ~(MemAlignment-1),PERM|IPC_CREAT))==-1) 
     {
      perror("could not allocate sharedmem buf");
      free(filename);
      if(strcmp(filename,"-")!=0) fclose(hFile);
      return -4;
     }
    // attach it
    if((ShMem=(MemForUse *)shmat(shmid,0,0))<0) 
     {
      perror ("could not attach sharedmem buf");
      free(filename);
      if(strcmp(filename,"-")!=0) fclose(hFile);
      if(shmctl(shmid,IPC_RMID,NULL)<0) perror("could not delete sharedmem buf");
      return -4;
     }

    if(hFile!=NULL)
     {
      if(hFile==stdin)
       {
        if(Objects!=0) 	
	 {
          short i;
	  unsigned long ulLen=0;
          for(i=0;i<Objects->getCount();i++)
	   {
	    _mempart *temp=(_mempart *)Objects->at(i);
            memcpy(&ShMem->buf[ulLen],temp->usBuf,temp->ulLen);
	    ulLen+=temp->ulLen;
	    free(temp->usBuf);
	    temp->usBuf=NULL;
	   } 

          Objects->shutDown();
          delete Objects;
	 } 
       }
      else 
       if(fread(ShMem->buf,1,ulFileLen,hFile)==0)
        {
         perror("could not read from file");
         dprintf(("Error in read(from %s %ld)\n",p,ulFileLen));
         free(filename);
         if(strcmp(filename,"-")!=0) fclose(hFile);
         if(shmdt(ShMem)==-1) perror ("could not detach sharedmem buf");
         if(shmctl(shmid,IPC_RMID,NULL)<0) perror ("could not delete sharedmem buf");
         return -3;
        }
      if(strcmp(filename,"-")!=0) fclose(hFile);
      dprintf(("From %s has been red %ld\n",p,ulFileLen));
     }
  
    ShMem->params.ShMKey=key;
    ShMem->params.ShMSize=ulFileLen;
    (void)sprintf(p,"<%x|%lx|>",key,ulFileLen);

    if(shmdt(ShMem)==-1)
     {
      perror ("could not detach sharedmem buf");
      return -4;
     }
   }
#endif
  
  ULONG ulFlags=0,ulDiffer=0;
  ret=AvpTestByDaemon(flags,tbuf,&ulFlags,&ulDiffer);
#ifdef USESHMEM
  if((flags&(FL_USESHMEM|FL_DOUNTUSESEM))!=0)
   {
    if((ulFlags&1)!=0)
     {
      if((shmid=shmget(key,ulDiffer+sizeof(MemForUse),0))<0)
       {
        dprintf(("could not call shmget(%ld)\n",ulDiffer+sizeof(MemForUse)));
        free(filename);
        return -4;
       }
      dprintf(("call shmget successfully (%ld bytes)",ulDiffer+sizeof(MemForUse))); 
      // attach it
      if((ShMem=(MemForUse *)shmat(shmid,0,0))<0) 
       {
        dprintf(("could not attach sharedmem buf"));
        free(filename);
        if(shmctl(shmid,IPC_RMID,NULL)) dprintf(("could not delete sharedmem buf"));
        return -5;
       }
      dprintf(("sharedmem attached successfully."));

      if((strlen(filename)!=0) && (strcmp(filename,"=")!=0))
       {
        int hFile;
        if((hFile=open(filename,O_WRONLY))==(-1))
         {
          dprintf(("Error in open(%s)\n",filename));
          free(filename);
          if(shmdt(ShMem)==-1) perror ("could not detach sharedmem buf");
          if(shmctl(shmid,IPC_RMID,NULL)<0) perror ("could not delete sharedmem buf");
          return -3;
         }
        lseek(hFile,0,SEEK_SET);
        if(write(hFile,ShMem->buf,ulDiffer)==-1)
          dprintf(("Error in write to %s %ld bytes\n",filename,ulDiffer));
        ftruncate(hFile,ulDiffer);
        close(hFile);
        printf("File changed successfully\n");
       }
      else
       printf("%s\n",ShMem->buf);
      free(filename);

      if(shmdt(ShMem)==-1) dprintf(("could not detach sharedmem buf"));
      // delete objects  
      if(shmctl(shmid,IPC_RMID,0)<0) dprintf(("umpossible delete shmem buf"));
     }
   }
#endif
  switch (ret)
   {
    case -1:
      printf("Error: cannot write to AvpDaemon!\n");
      break;
    case -2:
      printf("Error: cannot read from AvpDaemon!\n");
      break;
#ifdef USESHMEM
    case -4:
      printf("Error: error in shared memory block!\n");
      break;
#endif
   }
  va_end(ap);
  return ret;
 }

int AvpTest(int flags,const char *Name)
 {
  int ret=-1;

  if(AvpTestStart()==0)
   {
    if(flags==S_IFREG)
     {
#ifdef USESHMEM
      ret=AvpTest1(FL_USESHMEM|FL_DOUNTUSESEM,Name);
#else
      ret=AvpTest1(0,Name);
#endif
     }
    else
     if(flags==S_IFDIR) ret=AvpTest1(0,Name);      
     else
      printf("Error flag for object %s\n",Name);  
   }
  else
   {
    printf("I cannot connect to AvpDaemon and test %s!\n",Name);
   }    

  AvpTestClose();

  return (ret);
 }

