/* TeamDream -- read file(s) from standard output and check it(or they).
 * Return status code.
 * Copyright (C) 2000 Kaspersky Lab.

 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef linux
 #include <malloc.h>
#else
 #include <sys/stat.h> 
#endif
#include <getopt.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include "system.h"
//#include "error.h"
#include "debug.h"
#include "check.h"
#include "util.h"
#include "workstat.h"

// The official name of this program.
#ifdef linux
 #define AVP_NAME    "Antiviral Toolkit Pro for Linux (Cpp client for Daemon)"
#else
  #ifdef __FreeBSD__ 
   #define AVP_NAME    "Antiviral Toolkit Pro for FreeBSD (Cpp client for Daemon)"
  #else    
   #if defined (sun) && defined(i386)
    #define AVP_NAME    "Antiviral Toolkit Pro for Solaris (Cpp client for Daemon)"
   #else    
    #define AVP_NAME    "Antiviral Toolkit Pro for Unix (Cpp client for Daemon)"
   #endif
  #endif
#endif
#define AVP_DATE "built on "__DATE__" at "__TIME__ " "

// Name under which this program was invoked.  
char *ProcPath,*ProcName;

void usage(int status)
 {
  printf("Usage: %s [FILE]...",ProcName);
  printf("Send FILE(s), or standard input, to daemon and return status code.\n" \
         "With no FILE, or when FILE is -, read standard input.\n" \
         "  --help      display this help and exit\n"\
         "  --version   output version information and exit\n");
  puts("\nReport bugs to <support@avp.ru>.");

  exit (status==0 ? EXIT_SUCCESS:EXIT_FAILURE);
 }

int WorkByType(char *szObjName,int flags);

int main(int argc, char **argv)
 {	 
  int status=0; 

  if(*argv[0]=='/') ProcPath=getcpy(*argv,".");
  else              ProcPath=GetProcessFullname(NULL,0);
  ProcName=GetNameFromPath(ProcPath);
  if(ProcName!=ProcPath)
   ProcPath[ProcName-ProcPath-1]=0;
  ProcName=getcpy(ProcPath,"");

  dprintf(("Program starts in directory %s with name %s\n",ProcPath,ProcName));  

  static struct option const long_options[] =
   {
    {GETOPT_HELP_OPTION_DECL},
    {NULL, 0, NULL, 0}
   };

  // Parse command line options.  
  int c; //Index in argv to processed argument.  
  while ((c=getopt_long(argc,argv,"",long_options,NULL))!=-1)
   {
    switch (c)
     {
      case 0:
	break;
	case_GETOPT_HELP_CHAR;
      default:
	usage (EXIT_FAILURE);
     }
   }

  if(AvpTestStart()!=0)
   {
    RunFileWithFind("AvpDaemon",NULL,NULL);
    if(AvpTestStart()!=0)
     {
      printf("I can't run AvpDaemon. Program will be terminated\n");
      goto mQuit;
     } 
   }
  AvpTestClose();
  
  IniWorkStat();

  int objNumber; // number of checked objects
  objNumber=0;
  // Main loop.  
  if(argc<=1)
   {
    status=WorkByType("-",0);
    objNumber++;
   }
  else
  for(int arg=1;arg<argc;arg++)
   {
    if(argv[arg][0]=='-' && argv[arg][1]==0)
     {
      status=WorkByType("-",0);
      objNumber++;
     }
    else
     {
      status=WorkByType(argv[arg],0);
      objNumber++;
     }
   }

mQuit:
  free(ProcName);
  free(ProcPath);

  if(objNumber>1)
   {
    printf("You chech more then one object. Common result may be noncorrect.\n");
    status=ReportWorkStat(NULL);
   }
  else     
   fprintf(stdout,"Return code: %d\n",status);

  return status;
 }
