#ifdef linux
 #include <malloc.h>
#else
 #include <sys/stat.h> 
#endif
#include <fcntl.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/param.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <errno.h>
#ifndef errno
extern int errno;
#endif
#include "debug.h"

char *getcpy(char *str,char *defStr)
 {
  char *cp;
  size_t len;

  if(str!=NULL)
   {
    len=strlen(str)+1;
    if(!(cp=(char*)malloc(len+1))) dprintf(("Unable to allocate string storage\n"));
    memcpy(cp,str,len);
   }
  else
   {
    len=strlen(defStr)+1;
    if(!(cp=(char*)malloc((size_t)len))) dprintf(("unable to allocate string storage for default string"));
    memcpy(cp,defStr,len);
    // *cp='\0';
   }
  return cp;
 }

// Cool function (very Linux and /proc FS specific) to return full name
// of runned process
char* GetProcessFullname(char *szStartPath,unsigned int len)
 {
  char for_name[PATH_MAX + 1];
  struct stat name_stat;
  int depth = 0;
  
  //    /proc/my_pid/exe   . 
  sprintf(for_name, "/proc/%d/exe", getpid() );
  if(readlink(for_name, for_name, PATH_MAX)==-1) return NULL;
  while(depth < MAXSYMLINKS)
   {
    if(stat(for_name,&name_stat)==-1) return NULL;
    if(S_ISLNK(name_stat.st_mode) )
     {
      if(readlink(for_name,for_name,PATH_MAX)-1) return NULL;
      depth++;
      continue;
     }
    if(S_ISREG(name_stat.st_mode) )
     {
      if(szStartPath==NULL)
       return strdup(for_name);
      else
       { 
        strncpy(szStartPath,for_name,len);
        szStartPath[len-1]=0;
        return szStartPath;
       }	
     } 
    break;
   }
  return NULL; // exceeded max symlinks count 
 }

char * GetNameFromPath(char *lpFileName) // address of name of file to find path for
 {
  char *lpFilePart;
  lpFilePart=strrchr(lpFileName,'/');
  if(lpFilePart) (lpFilePart)++;
  else           lpFilePart=lpFileName; 
  return lpFilePart;
 }
 
int RunFile(char *path,char *arg,char *arg1)
 {
  dprintf(("%s trying to run with args: %s\n",path,arg1));

  int cpid, status;
  cpid = fork();
  switch (cpid)
    {
    case -1:			// Cannot fork.  
      return -1;		// errno is already set.  
    case 0:			// Child process.  
      char *name;
      name=GetNameFromPath(path);
      execl(path,name,arg,arg1,(char *)0);
      _exit (1);
    default:			// Parent process.  
      // Wait for kid to finish.  
      while (wait (&status) != cpid) ;// Do nothing.

      if(status & 0xFFFF)
	{
	  // run failed.  
	  errno = EIO;
	  return -1;
	}
      return 0;
    }
 }

char* which(char *file)
 {
  char *Path;		// user's $PATH
  if((Path=getenv("PATH"))) Path=getcpy(Path,"");

  if(Path!=NULL)
   {
    dprintf(("Your $PATH: %s\n",Path));
    char  *curpath=NULL;
    curpath=getcwd(NULL,PATH_MAX);
    
    if(Path[strlen(Path)-1]==':') Path[strlen(Path)-1]=0;
    char *span;
    span=strtok((char*)Path,":");
    while(span)
     {
      if(chdir(span)==0)
       {
        dprintf(("subpath: %s %s\n",span,file));
        if(open(file,O_RDONLY)!=-1)
         {
	  char *fillname=(char *)malloc(strlen(span)+strlen(file)+2);
	  strcpy(fillname,span);
          if(fillname[strlen(fillname)-1]!='/') strcat(fillname,"/");
	  strcat(fillname,file);
          dprintf(("fullname: %s\n",file));
          chdir(curpath);
	  return fillname;
         }
       }    
      span=strtok(NULL,":");
     }
    chdir(curpath);
   }
  return NULL;   
 }

extern char *ProcPath;
int RunFileWithFind(char *file,char *arg,char *arg1)
 {
  int rezcode=0;
  struct stat stbf;
  if(stat(file,&stbf)!=-1) return rezcode=RunFile(file,arg,arg1);

  char *tFold;
  tFold=(char *)malloc(strlen(file)+3);
  strcpy(tFold,"./");
  strcat(tFold,file);
  if(stat(tFold,&stbf)!=-1)
   {
    rezcode=RunFile(tFold,arg,arg1);
    free(tFold);
   }    
  else
   {
    free(tFold);
    tFold=(char *)malloc(strlen(ProcPath)+strlen(file)+2);
    strcpy(tFold,ProcPath);
    if(tFold[strlen(tFold)-1]!='/') strcat(tFold,"/");
    strcat(tFold,file);
    if(stat(tFold,&stbf)!=-1)
     {
      rezcode=RunFile(tFold,arg,arg1);
      free(tFold);
     } 
    else
     if((tFold=which(file))!=NULL)
      {
       dprintf(("RunFile file %s found as %s\n",file,tFold));
       rezcode=RunFile(tFold,arg,arg1);
       free(tFold);
      } 
     else printf("I cannot find %s\n",file);
   }  
  return rezcode;
 }
 
